// MOTDDisplayer
// Actor that calls the MOTD Menu.
class MOTDDisplayer extends Info
	config(MaDUtils);

// Server information. Defined in P2MOTD.ini
var config Texture ServerLogo;
var config string MOTDLine1;
var config string MOTDLine2;
var config string MOTDLine3;
var config string MOTDLine4;
var config string AdminName;
var config string AdminEmail;
var config string ServerName;

var Texture ThisServerLogo;
var string ThisMOTDLine1;
var string ThisMOTDLine2;
var string ThisMOTDLine3;
var string ThisMOTDLine4;
var string ThisAdminName;
var string ThisAdminEmail;
var string ThisServerName;

// Struct to record which MOTD's we've seen recently
struct SeenItStruct
{
	var config string TheMD5;
	var config int LastHours;
};

// Array of MOTD's we've seen recently
var config array<SeenItStruct> SeenIt;

var PlayerController PlayerOwner;
var PlayerController PlayerLocal;

// Replication block
replication
{
	// Things the server needs to send to the client
	reliable if (Role == ROLE_Authority && bNetInitial)
		ThisServerLogo, ThisServerName, ThisMOTDLine1, ThisMOTDLine2, ThisMOTDLine3, ThisMOTDLine4, ThisAdminName, ThisAdminEmail;
}

// SeenItYet
// Function that records what MOTD we're looking at.
// Returns true if we've seen it recently (within 1 hour), false if not.
simulated function bool SeenItYet()
{
	local string ConvMD5;
	local int i;
	local int CurrentHours;

	// Converts the server info to a nice MD5 we can use for comparison
	ConvMD5 = class'P2MD5'.Static.MD5String(ServerName@MOTDLine1@MOTDLine2@MOTDLine3@MOTDLine4@AdminName@AdminEmail);
	
	// Calculate the current time in hours
	CurrentHours = Level.Day*24 + Level.Hour;

	// Compare with recent MOTDs
	for (i=0; i<SeenIt.Length; i++)
		if (SeenIt[i].TheMD5 == ConvMD5)
			// We've seen it, but how long ago?
			if (CurrentHours < SeenIt[i].LastHours || (CurrentHours - SeenIt[i].LastHours > 1))
			{
				// If over 1 hour, re-display.
				SeenIt[i].LastHours = CurrentHours;
				return false;
			}
			else
				// If under 1 hour, skip the MOTD.
				return true;

	// Haven't seen it, so add the info.
	SeenIt.Insert(0, 1);
	SeenIt[0].TheMD5 = ConvMD5;
	SeenIt[0].LastHours = CurrentHours;
	SaveConfig();
	return false;
}

// Send server information to the MOTD menu.
simulated function Initialize(MenuMOTD MyMenu)
{
	MyMenu.ServerLogo = ThisServerLogo;
	MyMenu.MOTDLine1 = ThisMOTDLine1;
	MyMenu.MOTDLine2 = ThisMOTDLine2;
	MyMenu.MOTDLine3 = ThisMOTDLine3;
	MyMenu.MOTDLine4 = ThisMOTDLine4;
	MyMenu.AdminName = ThisAdminName;
	MyMenu.AdminEmail = ThisAdminEmail;
	MyMenu.ServerName = ThisServerName;
}

// GetLocalPlayer
// Returns the PlayerController of this client.
simulated function GetLocalPlayer()
{
	local PlayerController C;

	if (Level.NetMode != NM_DedicatedServer)
	{
		foreach DynamicActors(class 'PlayerController', C)
    		if (Viewport(C.Player) != None)
    		{
    			PlayerLocal = C;
      			break;
      		}
	}
}

simulated event PreBeginPlay()
{
	if (Role == ROLE_Authority)
	{
		ThisServerLogo = ServerLogo;
		ThisMOTDLine1 = MOTDLine1;
		ThisMOTDLine2 = MOTDLine2;
		ThisMOTDLine3 = MOTDLine3;
		ThisMOTDLine4 = MOTDLine4;
		ThisAdminName = AdminName;
		ThisAdminEmail = AdminEmail;
		ThisServerName = ServerName;
	}

	Super.PreBeginPlay();
}

// PostNetBeginPlay
// After our server info's been sent to the client, show them the MOTD.
simulated event PostNetBeginPlay()
{
	local MOTDDisplayer D;

	// Only display on clients.
	if (Level.NetMode != NM_DedicatedServer)
	{
		// Check to see if this message has been displayed recently
		if (!SeenItYet())
		{
			// If not, show the MOTD
			GetLocalPlayer();
			ClientShowMOTD(PlayerLocal);
		}
	}

	// Record our PlayerOwner
	PlayerOwner = PlayerController(Owner);
}

// Show the MOTD.
simulated event ClientShowMOTD(PlayerController PC)
{
	local ShellRootWindow RootMenu;

	RootMenu = ShellRootWindow(PC.Player.InteractionMaster.BaseMenu);
	RootMenu.JumpToMenu(class'MenuMOTD');
}

defaultproperties
{
     ServerLogo=Texture'MaDUtilsTex.MaDHeAdZServerLogo00'
     MOTDLine1="Let the Bullets Fly, and the Widows Cry!"
     MOTDLine2="No Pain, No Game!"
     MOTDLine3="No Risc, No Fun!"
     MOTDLine4="Knowledge is Power!"
     AdminName="-=M@DJ@cK@L=-"
     AdminEmail="madjackal_x@hotmail.com"
     ServerName="-[*_*]-={ThE M@DNeSs Nr:1}=-[*_*]-"
     RemoteRole=ROLE_SimulatedProxy
}
