//=============================================================================
// MaDTagSprayWeapon.
// Created on: 05/04/2005
// Written by Eduard Klemens
// Mail to: madjackal_x@hotmail.com.
//  2005, MaD Inc. All Rights Reserved.
//=============================================================================
class MaDTagSprayWeapon extends P2Weapon;

#exec OBJ LOAD FILE=MaDUtilsSounds.uax
#exec OBJ LOAD FILE=MaDUtilsTex.utx
#exec OBJ LOAD FILE=MaDUtilsAnim.ukx

var MaDTSprayGun MSMP;

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        MSMP = Spawn(Class'MaDUtils.MaDTSprayGun');
        AttachToBone(MSMP, 'TagGun');
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             MSMP.Destroyed();
             MSMP = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);


     MSMP.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( MSMP != None )
  {
    DetachFromBone(MSMP);
    MSMP.Destroy();
    MSMP = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Fire the weapon
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
{
	// Basics
	IncrementFlashCount();
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		// Play your sound in MP in your attachment to save bandwidth
		PlayAnim('Shoot1', WeaponSpeedShoot1 + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}
	else
	{
		Instigator.PlaySound(FireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
		PlayAnim('Shoot1', WeaponSpeedShoot1 + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}

	// Adv.
	SetupMuzzleFlash();

}

function ProjectileFire()
{
	if(Level.Game == None
		|| !Level.Game.bIsSinglePlayer)
//		|| (Instigator != None
//			&& PersonController(Instigator.Controller) != None))
	{
		SpawnTagProjectile(false);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Tell ammo to make this scissors type
///////////////////////////////////////////////////////////////////////////////
function SpawnTagProjectile(bool bMakeSpinner)
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor Other;
	local MaDTagProjectile tp;
	local P2Player p2p;

	if(AmmoType != None
		&& AmmoType.HasAmmo())
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z);
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		if(bMakeSpinner)
		{
			TurnOffHint();
			tp = spawn(class'MaDTagProjectile',Instigator,,StartTrace, AdjustedAim);
		}
		else
			tp = spawn(class'MaDTagProjectile',Instigator,,StartTrace, AdjustedAim);
		// Make sure it got made
		if(tp != None)
			P2AmmoInv(AmmoType).UseAmmoForShot();
	}
}

///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=150;
	LightHue=12+FRand()*15;
	LightRadius=12+FRand()*6;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure the muzzle flash renders, and move it around some
///////////////////////////////////////////////////////////////////////////////
simulated function SetupMuzzleFlash()
{
	bMuzzleFlash = true;
	bSetFlashTime = false;
	// Gets turned off in weapon, in RenderOverlays
	// Slightly change colors each time
	if(IsFirstPersonView()
		&& bDrawMuzzleFlash)
	{
	    PickLightValues();
		bDynamicLight=bAllowDynamicLights;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     bUsesAltFire=False
     ViolenceRank=0
     RecognitionDist=900.000000
     AI_BurstCountExtra=3
     AI_BurstTime=0.800000
     ShotMarkerMade=Class'Postal2Game.GunfireMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     holdstyle=WEAPONHOLDSTYLE_Single
     switchstyle=WEAPONHOLDSTYLE_Single
     firingstyle=WEAPONHOLDSTYLE_Single
     MinRange=250.000000
     ShakeOffsetTime=2.200000
     CombatRating=0.000000
     FirstPersonMeshSuffix="Pistol"
     WeaponSpeedLoad=1.500000
     WeaponSpeedReload=1.500000
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1Rand=0.040000
     bCanThrowMP=False
     AmmoName=Class'MaDUtils.MaDTagSprayAmmoInv'
     AutoSwitchPriority=0
     ShakeRotMag=(X=220.000000,Y=30.000000,Z=30.000000)
     ShakeRotTime=2.200000
     ShakeOffsetMag=(X=10.000000,Y=2.000000,Z=2.000000)
     TraceAccuracy=0.020000
     aimerror=400.000000
     AIRating=0.200000
     MaxRange=1024.000000
     FireSound=Sound'MaDUtilsSounds.MaDTagFire'
     SelectSound=Sound'MaDUtilsSounds.MaDTagSelect'
     FlashOffsetY=0.082000
     FlashOffsetX=0.220000
     FlashLength=0.010000
     MuzzleFlashSize=128.000000
     MFTexture=Texture'MaDUtilsTex.MaDTagMuzzleFTex'
     bDrawMuzzleFlash=True
     MuzzleFlashScale=3.400000
     MuzzleFlashStyle=STY_Normal
     GroupOffset=20
     PickupClass=Class'MaDUtils.MaDTagSprayPickup'
     PlayerViewOffset=(X=0.000000,Z=-1.000000)
     BobDamping=0.975000
     AttachmentClass=Class'MaDUtils.MaDTagSprayAttachment'
     ItemName="M@D Tag Spray Gun"
     Mesh=SkeletalMesh'MaDUtilsAnim.MaD_LS_Tag'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     AmbientGlow=64
     SoundRadius=255.000000
}
