// MenuMOTD
// A menu that displays the server's logo and the message of the day.
class MenuMOTD extends ShellMenuCW;

// Pointer to the MOTDDisplayer responsible for this menu
var MOTDDisplayer MyDisplayer;

// Localization
var localized string Dismiss;
var localized string DismissHelp;
var localized string WelcomeTo;
var localized string YourAdminIs;

// "Dismiss" choice
var ShellMenuChoice CancelChoice;

// Server information. Sent from the MOTDDisplayer
var Texture ServerLogo;
var string ServerName;
var string AdminName;
var string AdminEmail;
var string MOTDLine1;
var string MOTDLine2;
var string MOTDLine3;
var string MOTDLine4;

// "Choices" for the MOTD lines, welcome text, etc.
var ShellMenuChoice MOTDChoice1;
var ShellMenuChoice MOTDChoice2;
var ShellMenuChoice MOTDChoice3;
var ShellMenuChoice MOTDChoice4;
var ShellMenuChoice MOTDTitleChoice;
var ShellMenuChoice WelcomeChoice;

// GetMyDisplayer
// Find the MOTDDisplayer responsible for this menu.
function MOTDDisplayer GetMyDisplayer()
{
	local MOTDDisplayer D;

	foreach GetLevel().DynamicActors(class'MOTDDisplayer', D)
			return D;
}

// CreateMenuContents
// Called by shell to put up the menu's contents.
function CreateMenuContents()
{
	local int i;

	// Call super
	Super.CreateMenuContents();

	// Find and initialize our MOTDDisplayer. This sets our variables
	MyDisplayer = GetMyDisplayer();
	MyDisplayer.Initialize(Self);

	if (MyDisplayer == None)
		warn("Couldn't find my MOTD Displayer!");

	// Server Logo
	if (ServerLogo != None)
		AddTitleBitmap(ServerLogo);

	// Welcome to (server name)
	if (ServerName != "")
	{
		MOTDTitleChoice = AddChoice(WelcomeTo @ ServerName, "", F_FancyM, TA_Center);
		MOTDTitleChoice.bActive = False;
	}

	// Your admin is Admin (admin@gopostal.com)
	if (AdminName != "")
	{
		if (AdminEmail == "")
			WelcomeChoice = AddChoice(YourAdminIs @ AdminName, "", F_FancyM, TA_Center);
		else
			WelcomeChoice = AddChoice(YourAdminIs @ AdminName @ "(" $ AdminEmail $ ")", "", F_FancyM, TA_Center);
		WelcomeChoice.bActive = False;
	}

	// MOTD lines
	if (MOTDLine1 != "")
	{
		MOTDChoice1 = AddChoice(MOTDLine1, "", F_FancyS, TA_Center);
		MOTDChoice1.bActive = False;
	}
	if (MOTDLine2 != "")
	{
		MOTDChoice2 = AddChoice(MOTDLine2, "", F_FancyS, TA_Center);
		MOTDChoice2.bActive = False;
	}
	if (MOTDLine3 != "")
	{
		MOTDChoice3 = AddChoice(MOTDLine3, "", F_FancyS, TA_Center);
		MOTDChoice3.bActive = False;
	}
	if (MOTDLine4 != "")
	{
		MOTDChoice4 = AddChoice(MOTDLine4, "", F_FancyS, TA_Center);
		MOTDChoice4.bActive = False;
	}

	// Dismiss MOTD
	CancelChoice = AddChoice(Dismiss, DismissHelp, F_FancyS, TA_Left, True);
}

function HideThisMenu()
{
	GoBack();
	ShellRootWindow(Root).HideMenu();
}

function ReturnPlayerToGame()
{
	HideThisMenu();
}

///////////////////////////////////////////////////////////////////////////////
// Handle notifications from controls
///////////////////////////////////////////////////////////////////////////////
function Notify(UWindowDialogControl C, byte E)
{
	Super.Notify(C, E);
	switch(E)
	{
		case DE_Click:
			if (C != None)
				switch (C)
				{
					case CancelChoice:
						ReturnPlayerToGame();
				}
			break;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Handle a key event
///////////////////////////////////////////////////////////////////////////////
function bool KeyEvent( out EInputKey Key, out EInputAction Action, FLOAT Delta )
{
	local ShellMenuChoice choice;
	local int i;

	if (Action == IST_Release)
	{
		switch (Key)
		{
			case IK_ESCAPE:
				ReturnPlayerToGame();
				return true;
		}
	}

	return Super.KeyEvent(Key, Action, Delta);
}

defaultproperties
{
     Dismiss="Cancel"
     DismissHelp="Cancel this window and enter the M@DNeSs already!"
     WelcomeTo="Welcome to"
     YourAdminIs="Your -={MH}=- Admin is"
     MenuWidth=1024.000000
     TitleHeight=512.000000
     astrTextureDetailNames(0)="UltraLow"
     astrTextureDetailNames(1)="Low"
     astrTextureDetailNames(2)="Medium"
     astrTextureDetailNames(3)="High"
}
