//=============================================================================
// MutMaDAntiCamper.
// Created on: 02/25/2005
// Written by Eduard Klemens
// AntiCamper v1.0 by Shawn "Hanzo" Holmes
// based on a UT Mutator by Patrick Freeman
// Mail to: madjackal_x@hotmail.com.
//  2005, MaD Inc. All Rights Reserved.
//=============================================================================
class MutMaDAntiCamper extends Mutator
    config(MaDUtils);

var() config int CampTime;
var() config int CampRadius;
var() config int CampDamage;

struct PInfo
{
	var int		PlayerID;
	var vector	PlayerLoc;
};

var array<PInfo> OldPlayer;
var array<PInfo> CurrentPlayer;
var float CurrentCampTime;

event PreBeginPlay()
{
	CurrentCampTime = 0.0;
	Enable('Tick');
}

function Tick(float DeltaTime)
{
	CurrentCampTime += DeltaTime;
	if (CurrentCampTime > CampTime)
	{
		UpdateAllPlayers();
		CurrentCampTime = 0.0;
	}
}

function UpdateAllPlayers()
{
    local Controller C;
	local Pawn PawnLink;
    local int i, j;

	CurrentPlayer.Remove(0,CurrentPlayer.length);

	for (C = Level.ControllerList; C != None; C = C.NextController)
	{
        PawnLink = C.Pawn;
		if ( PawnLink != None && C.bIsPlayer && !PawnLink.PlayerReplicationInfo.bOnlySpectator )
		{
	        i = CurrentPlayer.Length;
	        CurrentPlayer.Insert(i,1);
			CurrentPlayer[i].PlayerID = PawnLink.PlayerReplicationInfo.PlayerID;
			CurrentPlayer[i].PlayerLoc = PawnLink.Location;
		}
	}

    for (j=0; j < CurrentPlayer.length; j++)
	{
		for (i=0; i < OldPlayer.length; i++)
		{
			if (OldPlayer[i].PlayerID == CurrentPlayer[j].PlayerID)
			{
				CheckForCamper(OldPlayer[i], CurrentPlayer[j]);
    			break; // this player was found, move to next in "CurrentPlayer" list.
			}
		}
	}

	OldPlayer.Remove(0,OldPlayer.length);
	for (i=0; i<CurrentPlayer.length; i++)
	{
	   OldPlayer.Insert(i,1);
	   OldPlayer[i].PlayerID	= CurrentPlayer[i].PlayerID;
	   OldPlayer[i].PlayerLoc	= CurrentPlayer[i].PlayerLoc;
	}
}

function CheckForCamper(PInfo pLast, PInfo pCurrent)
{
    local Controller C;
	local Pawn PawnLink;
	local float VDistance;
	local Vector DVector;
	local string msg;
	local int i;

	DVector = pLast.PlayerLoc - pCurrent.PlayerLoc;
	VDistance = VSize(DVector);
	if (VDistance < CampRadius)
	{
        for (C = Level.ControllerList; C != None; C = C.NextController)
		{
    	    PawnLink = C.Pawn;
			if ( C.bIsPlayer )
			{
				if (PawnLink.PlayerReplicationInfo.PlayerID == pLast.PlayerID)
                    break;
			}
		}

		if (PawnLink != None)
		{
			PawnLink.Health -= CampDamage;

			if (PawnLink.Health <= 0)
			{
			    msg = "CAMPER: You have been Killed by M@DJ@cK@L for your behavior.";
				PawnLink.ClientMessage(msg);
				BroadcastLocalizedMessage( class'MaDCamperKillMessage', 0, PawnLink.PlayerReplicationInfo);
				PawnLink.Died(None, None, vect(0,0,0));
				// clear this player out of the pLast list
				// this is to prevent a possible incorrect "camper" tag
                // on a respawn'd player walking through the last valid camp zone
				for (i=0; i<OldPlayer.length; i++)
            	{
            	    if (OldPlayer[i].PlayerID == pCurrent.PlayerID)
            		{
            		    OldPlayer.Remove(i,1);
            		    break;
            		}
               	}
    			return;
			}

			if (CampDamage > 0)
			{
				msg = "CAMPER: You have been penalized by M@DJ@cK@L. Start Moving or Die.";
				PawnLink.ClientMessage(msg);
				BroadcastLocalizedMessage( class'MaDCamperAlert', 0, PawnLink.PlayerReplicationInfo);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function bool MutatorIsAllowed()
{
	return true;
}

defaultproperties
{
     CampTime=25
     CampRadius=300
     CampDamage=150
     GroupName="MaDAntiCamper"
     FriendlyName="M@DUtils: M@D Anti Camper"
     Description="M@D Inc. penalizes Campers by reducing health and alerting players."
}
